module Main where
import System.Environment (getArgs)
import Calendar
import Picture 

{-| 'rjustify n xs' right-justifies its String argument
'xs' inside a box of width 'n'. -}
rjustify :: Int -> String -> String
rjustify n xs = replicate (n - length xs) ' ' ++ xs

{-| 'group n xs' splits the list 'xs' into segments
of length 'n' (the last one may be shorter). -}
group :: Int -> [a] -> [[a]]
group n xs = if null ys then [] else ys : group n zs
  where
    (ys, zs) = splitAt n xs

daysOfMonth :: Month -> Year -> [Picture]
daysOfMonth m y =
  map (row . rjustify 3 . pic) [1-d..42-d]
  where
    (d, t) = monthInfo m y
    pic n = if 1 <= n && n <= t then show n else ""

month :: Month -> Year -> Picture
month m y = tile $ group 7 $ daysOfMonth m y

printMonth m y = putStr $ showPic $ above weekdays $ month m y
  where
    weekdays = row " Su Mo Tu We Th Fr Sa"

main = do
  args <- getArgs
  case args of
    [m, y] -> printMonth (read m) (read y)
    _     -> error "expecting 2 arguments (month and year)"
